@@ -2,17 +2,14 @@ package IPC::SRLock;
 
 use 5.010001;
 use namespace::autoclean;
-use version; our $VERSION = qv( sprintf '0.27.%d', q$Rev: 1 $ =~ /\d+/gmx );
+use version; our $VERSION = qv( sprintf '0.28.%d', q$Rev: 1 $ =~ /\d+/gmx );
 
 use File::DataClass::Types qw( HashRef LoadableClass NonEmptySimpleStr Object );
+use IPC::SRLock::Utils     qw( merge_attributes );
 use Moo;
 
 my $_build__implementation = sub {
-   my $self = shift; my $name = lc join '_', split m{ :: }mx, __PACKAGE__, -1;
-
-   my $attr = { name => $name, %{ $self->_implementation_attr }, };
-
-   return $self->_implementation_class->new( $attr );
+   return $_[ 0 ]->_implementation_class->new( $_[ 0 ]->_implementation_attr );
 };
 
 my $_build__implementation_class = sub {
@@ -32,8 +29,7 @@ has '_implementation'       => is => 'lazy', isa => Object,
    handles                  => [ qw( get_table list reset set ) ],
    builder                  => $_build__implementation;
 
-has '_implementation_attr'  => is => 'ro',   isa => HashRef,
-   default                  => sub { {} };
+has '_implementation_attr'  => is => 'ro',   isa => HashRef, required => 1;
 
 has '_implementation_class' => is => 'lazy', isa => LoadableClass,
    builder                  => $_build__implementation_class;
@@ -42,12 +38,20 @@ has '_implementation_class' => is => 'lazy', isa => LoadableClass,
 around 'BUILDARGS' => sub {
    my ($orig, $self, @args) = @_; my $attr = $orig->( $self, @args );
 
-   my $type = delete $attr->{type};
+   my $builder = $attr->{builder};
+   my $conf    = $builder && $builder->can( 'config' ) ? $builder->config : 0;
 
-   $attr = { _implementation_attr => $attr }; $type or return $attr;
+   $conf and $conf->can( 'lock_attributes' )
+         and merge_attributes $attr, $conf->lock_attributes,
+                           [ keys %{ $conf->lock_attributes } ];
 
-   if ($type =~ m{ \A ([a-zA-Z0-9\:\+]+) \z }mx) { $attr->{type} = $1 }
-   else { die "Type ${type} tainted" }
+   $attr->{name} //= lc join '_', split m{ :: }mx, __PACKAGE__, -1;
+
+   my $type = delete $attr->{type}; $attr = { _implementation_attr => $attr };
+
+   $type and $type !~ m{ \A ([a-zA-Z0-9\:\+]+) \z }mx
+         and die "Type ${type} tainted";
+   $type and $attr->{type} = $1;
 
    return $attr;
 };
@@ -79,7 +83,7 @@ IPC::SRLock - Set / reset locking semantics to single thread processes
 
 =head1 Version
 
-This documents version v0.27.$Rev: 1 $ of L<IPC::SRLock>
+This documents version v0.28.$Rev: 1 $ of L<IPC::SRLock>
 
 =head1 Synopsis
 
