@@ -7,7 +7,8 @@ use File::DataClass::Constants qw( LOCK_BLOCKING LOCK_NONBLOCKING );
 use File::DataClass::Types     qw( Directory NonEmptySimpleStr
                                    OctalNum Path PositiveInt RegexpRef );
 use File::Spec;
-use IPC::SRLock::Utils         qw( Unspecified hash_from loop_until throw );
+use IPC::SRLock::Utils         qw( Unspecified hash_from loop_until
+                                   merge_attributes throw );
 use Storable                   qw( nfreeze thaw );
 use Try::Tiny;
 use Moo;
@@ -61,6 +62,18 @@ has '_shmfile'       => is => 'lazy', isa => Path, coerce => 1,
 has '_shmfile_name'  => is => 'ro',   isa => NonEmptySimpleStr,
    init_arg          => 'shmfile';
 
+# Construction
+around 'BUILDARGS' => sub {
+   my ($orig, $self, @args) = @_; my $attr = $orig->( $self, @args );
+
+   my $builder = $attr->{builder} or return $attr;
+   my $config  = $builder->can( 'config' ) ? $builder->config : {};
+
+   merge_attributes $attr, $config, [ 'tempdir' ];
+
+   return $attr;
+};
+
 # Private methods
 my $_expire_lock = sub {
    my ($self, $content, $key, $lock) = @_;
@@ -95,27 +108,38 @@ my $_read_shmfile = sub {
    return ($file, $content);
 };
 
+my $_unlock_share = sub {
+   $_[ 0 ]->close; return 1;
+};
+
 my $_write_shmfile = sub {
-   my ($self, $file, $content) = @_; my $wtr;
+   my ($self, $lock_file, $content) = @_; my $wtr;
 
    try   { $wtr = $self->_shmfile->assert_open( 'w', $self->mode ) }
-   catch { $file->close; throw $_ };
+   catch { $_unlock_share->( $lock_file ); throw $_ };
 
    try   { $wtr->print( nfreeze $content ) }
-   catch { $wtr->delete; $file->close; throw $_ };
+   catch { $wtr->delete; $_unlock_share->( $lock_file ); throw $_ };
 
-   $wtr->close; $file->close;
-   return;
+   $wtr->close; $_unlock_share->( $lock_file );
+   return 1;
 };
 
 my $_reset = sub {
-   my ($self, $key) = @_;
-   my ($lock_file, $shm_content) = $self->$_read_shmfile;
-   my $found = delete $shm_content->{ $key };
+   my ($self, $args) = @_; my $key = $args->{k}; my $pid = $args->{p};
 
-   $found and $self->$_write_shmfile( $lock_file, $shm_content );
-   $lock_file->close; $found or throw 'Lock [_1] not set', [ $key ];
-   return 1;
+   my ($lock_file, $shm_content) = $self->$_read_shmfile;
+
+   my $lock; exists $shm_content->{ $key }
+      and $lock = $shm_content->{ $key }
+      and $lock->{spid} != $pid
+      and $_unlock_share->( $lock_file )
+      and throw 'Lock [_1] set by another process', [ $key ];
+
+   not delete $shm_content->{ $key } and $_unlock_share->( $lock_file )
+      and throw 'Lock [_1] not set', [ $key ];
+
+   return $self->$_write_shmfile( $lock_file, $shm_content );
 };
 
 my $_set = sub {
@@ -123,24 +147,21 @@ my $_set = sub {
 
    my ($lock_file, $shm_content) = $self->$_read_shmfile( $args->{async} );
 
-   if ($lock_file->have_lock) {
-      my $lock; exists $shm_content->{ $key }
-         and $lock = $shm_content->{ $key }
-         and $lock->{timeout}
-         and $now > $lock->{stime} + $lock->{timeout}
-         and $lock = $self->$_expire_lock( $shm_content, $key, $lock );
+   not $lock_file->have_lock and $_unlock_share->( $lock_file ) and return 0;
 
-      unless ($lock) {
-         $shm_content->{ $key }
-            = { spid => $pid, stime => $now, timeout => $args->{t} };
-         $self->$_write_shmfile( $lock_file, $shm_content );
-         $self->log->debug( "Lock ${key} set by ${pid}" );
-         return 1;
-      }
-   }
+   my $lock; exists $shm_content->{ $key }
+      and $lock = $shm_content->{ $key }
+      and $lock->{timeout}
+      and $now > $lock->{stime} + $lock->{timeout}
+      and $lock = $self->$_expire_lock( $shm_content, $key, $lock );
 
-   $lock_file->close;
-   return 0;
+   $lock and $_unlock_share->( $lock_file ) and return 0;
+
+   $shm_content->{ $key }
+      = { spid => $pid, stime => $now, timeout => $args->{t} };
+   $self->$_write_shmfile( $lock_file, $shm_content );
+   $self->log->debug( "Lock ${key} set by ${pid}" );
+   return 1;
 };
 
 # Public methods
@@ -160,7 +181,7 @@ sub list {
 }
 
 sub reset {
-   my $self = shift; return $self->$_reset( $self->_get_args( @_ )->{k} );
+   my $self = shift; return $self->$_reset( $self->_get_args( @_ ) );
 }
 
 sub set {
@@ -228,6 +249,11 @@ The umask to set when creating the lock table file. Defaults to 0
 
 =head1 Subroutines/Methods
 
+=head2 C<BUILDARGS>
+
+Extract the L</tempdir> attribute value from the C<config> object
+if one was supplied
+
 =head2 list
 
 List the contents of the lock table
